#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import json

# import interfaces
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IParagraphContainer
from pyams_content.component.paragraph.interfaces.keynumber import KEYNUMBER_PARAGRAPH_TYPE, IKeyNumberParagraph, \
    IKeyNumberContainer, IKeyNumberContainerTarget, IKeyNumber
from pyams_content.component.paragraph.zmi import IParagraphContainerView, IParagraphEditFormButtons
from pyams_content.component.paragraph.zmi.interfaces import IParagraphInnerEditor
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_content.shared.common import IWfSharedContent
from pyams_form.interfaces.form import IInnerForm, IInnerSubForm
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu, IWidgetTitleViewletManager
from pyams_skin.layer import IPyAMSLayer
from z3c.form.interfaces import INPUT_MODE
from z3c.table.interfaces import IValues, IColumn

# import packages
from pyams_content.component.paragraph.keynumber import KeyNumberParagraph, KeyNumber
from pyams_content.component.paragraph.zmi import BaseParagraphAddMenu, BaseParagraphAJAXAddForm, \
    BaseParagraphPropertiesEditForm, BaseParagraphAJAXEditForm
from pyams_content.features.renderer.zmi.widget import RendererFieldWidget
from pyams_form.form import AJAXAddForm, ajax_config
from pyams_form.security import ProtectedFormObjectMixin
from pyams_i18n.column import I18nAttrColumn
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.container import switch_element_visibility
from pyams_skin.event import get_json_widget_refresh_event, get_json_switched_table_refresh_event, \
    get_json_table_row_refresh_event
from pyams_skin.table import BaseTable, SorterColumn, I18nColumn, VisibilitySwitcherColumn, TrashColumn
from pyams_skin.viewlet.toolbar import ToolbarAction
from pyams_utils.adapter import adapter_config, ContextRequestViewAdapter
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm, AdminDialogEditForm
from pyams_zmi.zmi.table import InnerTableView
from pyramid.decorator import reify
from pyramid.view import view_config
from z3c.form import field, button
from z3c.table.column import GetAttrColumn
from zope.interface import implementer, Interface

from pyams_content import _


class IKeyNumbersView(Interface):
    """Key numbers view marker interface"""


class IKeyNumbersParentForm(Interface):
    """Key numbers parent form marker interface"""


@viewlet_config(name='add-keynumber-paragraph.menu', context=IParagraphContainerTarget, view=IParagraphContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=600)
class KeyNumberParagraphAddMenu(BaseParagraphAddMenu):
    """Key number paragraph add menu"""
    
    label = _("Key numbers...")
    label_css_class = 'fa fa-fw fa-dashboard'
    url = 'add-keynumber-paragraph.html'
    paragraph_type = KEYNUMBER_PARAGRAPH_TYPE
    
    
@pagelet_config(name='add-keynumber-paragraph.html', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
@ajax_config(name='add-keynumber-paragraph.json', context=IParagraphContainerTarget, layer=IPyAMSLayer,
             base=BaseParagraphAJAXAddForm)
class KeyNumberParagraphAddForm(AdminDialogAddForm):
    """Key number paragraph add form"""
    
    legend = _("Add new key number paragraph")
    icon_css_class = 'fa fa-fw fa-dashboard'
    
    fields = field.Fields(IKeyNumberParagraph).select('title', 'renderer')
    edit_permission = MANAGE_CONTENT_PERMISSION
    
    def create(self, data):
        return KeyNumberParagraph()
    
    def add(self, object):
        IParagraphContainer(self.context).append(object)

    
@pagelet_config(name='properties.html', context=IKeyNumberParagraph, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
@ajax_config(name='properties.json', context=IKeyNumberParagraph, layer=IPyAMSLayer,
             base=BaseParagraphAJAXEditForm)
class KeyNumberParagraphPropertiesEditForm(BaseParagraphPropertiesEditForm):
    """Key number paragraph properties edit form"""

    prefix = 'keynumbers_properties.'

    @property
    def title(self):
        content = get_parent(self.context, IWfSharedContent)
        return II18n(content).query_attribute('title', request=self.request)

    legend = _("Edit key number paragraph properties")
    icon_css_class = 'fa fa-fw fa-dashboard'

    fields = field.Fields(IKeyNumberParagraph).select('title', 'renderer')
    fields['renderer'].widgetFactory = RendererFieldWidget

    edit_permission = MANAGE_CONTENT_PERMISSION


@adapter_config(context=(IKeyNumberParagraph, IPyAMSLayer), provides=IParagraphInnerEditor)
@ajax_config(name='inner-properties.json', context=IKeyNumberParagraph, layer=IPyAMSLayer,
             base=BaseParagraphAJAXEditForm)
@implementer(IInnerForm, IKeyNumbersParentForm)
class KeyNumberParagraphInnerEditForm(KeyNumberParagraphPropertiesEditForm):
    """Key number paragraph inner edit form"""

    legend = None

    @property
    def buttons(self):
        if self.mode == INPUT_MODE:
            return button.Buttons(IParagraphEditFormButtons)
        else:
            return button.Buttons()

    def get_ajax_output(self, changes):
        output = super(self.__class__, self).get_ajax_output(changes)
        updated = changes.get(IKeyNumberParagraph, ())
        if 'renderer' in updated:
            output.setdefault('events', []).append(
                get_json_widget_refresh_event(self.context, self.request, KeyNumberParagraphInnerEditForm, 'renderer'))
        return output


#
# Key number items table view
#

class KeyNumbersTable(ProtectedFormObjectMixin, BaseTable):
    """Key numbers view inner table"""

    prefix = 'keynumbers'

    hide_header = True
    sortOn = None

    @property
    def cssClasses(self):
        classes = ['table', 'table-bordered', 'table-striped', 'table-hover', 'table-tight']
        permission = self.permission
        if (not permission) or self.request.has_permission(permission, self.context):
            classes.append('table-dnd')
        return {'table': ' '.join(classes)}

    @property
    def data_attributes(self):
        attributes = super(KeyNumbersTable, self).data_attributes
        attributes['table'] = {
            'id': self.id,
            'data-ams-location': absolute_url(IKeyNumberContainer(self.context), self.request),
            'data-ams-tablednd-drag-handle': 'td.sorter',
            'data-ams-tablednd-drop-target': 'set-keynumbers-order.json',
            'data-ams-visibility-switcher': 'switch-keynumber-visibility.json'
        }
        return attributes

    @reify
    def values(self):
        return list(super(KeyNumbersTable, self).values)


@adapter_config(context=(IKeyNumberContainerTarget, IPyAMSLayer, KeyNumbersTable), provides=IValues)
class KeyNumbersTableValuesAdapter(ContextRequestViewAdapter):
    """Key numbers table values adapter"""

    @property
    def values(self):
        return IKeyNumberContainer(self.context).values()


@adapter_config(name='sorter', context=(IKeyNumberContainerTarget, IPyAMSLayer, KeyNumbersTable), provides=IColumn)
class KeyNumbersTableSorterColumn(ProtectedFormObjectMixin, SorterColumn):
    """Key numbers table sorter column"""


@view_config(name='set-keynumbers-order.json', context=IKeyNumberContainer, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
def set_keynumbers_order(request):
    """Update key numbers order"""
    order = list(map(str, json.loads(request.params.get('names'))))
    request.context.updateOrder(order)
    return {'status': 'success'}


@adapter_config(name='show-hide', context=(IKeyNumberContainerTarget, IPyAMSLayer, KeyNumbersTable),
                provides=IColumn)
class KeyNumbersTableShowHideColumn(ProtectedFormObjectMixin, VisibilitySwitcherColumn):
    """Key numbers container visibility switcher column"""


@view_config(name='switch-keynumber-visibility.json', context=IKeyNumberContainer, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
def switch_keynumber_visibility(request):
    """Switch key number visibility"""
    return switch_element_visibility(request, IKeyNumberContainer)


@adapter_config(name='label', context=(IKeyNumberContainerTarget, IPyAMSLayer, KeyNumbersTable), provides=IColumn)
class KeyNumbersTableLabelColumn(I18nColumn, I18nAttrColumn):
    """Key numbers table label column"""

    _header = _('key-number-label', default="Header")
    attrName = 'label'
    weight = 10

    def getValue(self, obj):
        return super(KeyNumbersTableLabelColumn, self).getValue(obj) or '--'


@adapter_config(name='name', context=(IKeyNumberContainerTarget, IPyAMSLayer, KeyNumbersTable), provides=IColumn)
class KeyNumbersTableNameColumn(I18nColumn, GetAttrColumn):
    """Key numbers table number column"""

    _header = _("Number")
    attrName = 'number'
    weight = 20


@adapter_config(name='unit', context=(IKeyNumberContainerTarget, IPyAMSLayer, KeyNumbersTable), provides=IColumn)
class KeyNumbersTableUnitColumn(I18nColumn, I18nAttrColumn):
    """Key numbers table unit column"""

    _header = _('key-number-unit', default="Unit")
    attrName = 'unit'
    weight = 30

    def getValue(self, obj):
        return super(KeyNumbersTableUnitColumn, self).getValue(obj) or '--'


@adapter_config(name='text', context=(IKeyNumberContainerTarget, IPyAMSLayer, KeyNumbersTable), provides=IColumn)
class KeyNumbersTableTextColumn(I18nColumn, I18nAttrColumn):
    """Key numbers table text column"""

    _header = _("Associated text")
    attrName = 'text'
    weight = 40

    def getValue(self, obj):
        return super(KeyNumbersTableTextColumn, self).getValue(obj) or '--'


@adapter_config(name='trash', context=(IKeyNumberContainerTarget, IPyAMSLayer, KeyNumbersTable), provides=IColumn)
class KeyNumbersTableTrashColumn(ProtectedFormObjectMixin, TrashColumn):
    """Key numbers table trash column"""


@adapter_config(name='keynumbers', context=(IKeyNumberContainerTarget, IPyAMSLayer, IKeyNumbersParentForm),
                provides=IInnerSubForm)
@implementer(IKeyNumbersView)
class KeyNumbersView(InnerTableView):
    """Key numbers view"""

    title = _("Key numbers")

    table_class = KeyNumbersTable
    weight = 110


#
# Key numbers forms
#

@viewlet_config(name='add-keynumber.action', context=IKeyNumberContainerTarget, layer=IPyAMSLayer, view=IKeyNumbersView,
                manager=IWidgetTitleViewletManager, permission=MANAGE_CONTENT_PERMISSION, weight=1)
class KeyNumberAddAction(ToolbarAction):
    """Key number add action"""

    label = _("Add keynumber")
    label_css_class = 'fa fa-fw fa-plus'
    url = 'add-keynumber.html'
    modal_target = True


@pagelet_config(name='add-keynumber.html', context=IKeyNumberContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
@ajax_config(name='add-keynumber.json', context=IParagraphContainerTarget, layer=IPyAMSLayer,
             base=AJAXAddForm)
class KeyNumberAddForm(AdminDialogAddForm):
    """Key number add form"""

    legend = _("Add new keynumber")
    icon_css_class = 'fa fa-fw fa-dashboard'

    fields = field.Fields(IKeyNumber).omit('__parent__', '__name__', 'visible')
    edit_permission = MANAGE_CONTENT_PERMISSION

    def create(self, data):
        return KeyNumber()

    def add(self, object):
        IKeyNumberContainer(self.context).append(object)

    def get_ajax_output(self, changes):
        return {
            'status': 'success',
            'message': self.request.localizer.translate(_("Key number was correctly added")),
            'events': [get_json_switched_table_refresh_event(self.context, self.request, KeyNumbersTable), ]
        }


@pagelet_config(name='properties.html', context=IKeyNumber, layer=IPyAMSLayer, permission=MANAGE_CONTENT_PERMISSION)
@ajax_config(name='properties.json', context=IKeyNumber, layer=IPyAMSLayer)
class KeyNumberPropertiesEditForm(AdminDialogEditForm):
    """Key number properties edit form"""

    prefix = 'keynumber_properties.'

    legend = _("Edit keynumber properties")
    icon_css_class = 'fa fa-fw fa-dashboard'

    fields = field.Fields(IKeyNumber).omit('__parent__', '__name__', 'visible')
    edit_permission = MANAGE_CONTENT_PERMISSION

    def get_ajax_output(self, changes):
        output = super(self.__class__, self).get_ajax_output(changes)
        updated = changes.get(IKeyNumber, ())
        if updated:
            target = get_parent(self.context, IKeyNumberContainerTarget)
            output.setdefault('events', []).append(
                get_json_table_row_refresh_event(target, self.request, KeyNumbersTable, self.context))
        return output
