#
# -*- encoding: utf-8 -*-
#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.illustration import IIllustrationTarget
from pyams_content.features.alert.interfaces import IAlertTarget
from pyams_content.features.footer.interfaces import IFooterTarget
from pyams_content.features.header.interfaces import IHeaderTarget
from pyams_content.features.preview.interfaces import IPreviewTarget
from pyams_content.interfaces import WEBMASTER_ROLE, OPERATOR_ROLE
from pyams_content.root.interfaces import ISiteRootRoles, ISiteRootConfiguration, ISiteRoot, \
    ISiteRootToolsConfiguration, ISiteRootBackOfficeConfiguration
from pyams_form.interfaces.form import IFormContextPermissionChecker
from pyams_portal.interfaces import IPortalContext
from pyams_security.interfaces import IDefaultProtectionPolicy, IGrantedRoleEvent, ISecurityManager
from pyams_utils.interfaces import MANAGE_SYSTEM_PERMISSION
from pyams_utils.interfaces.site import IConfigurationFactory, IBackOfficeConfigurationFactory, ISiteRootFactory

# import packages
from persistent import Persistent
from pyams_security.property import RolePrincipalsFieldProperty
from pyams_security.security import ProtectedObject
from pyams_skin.configuration import Configuration, BackOfficeConfiguration
from pyams_skin.skin import UserSkinnableContent
from pyams_utils.adapter import adapter_config, ContextAdapter, get_annotation_adapter
from pyams_utils.registry import get_utility, utility_config
from pyams_utils.site import BaseSiteRoot
from pyams_utils.traversing import get_parent
from pyramid.events import subscriber
from zope.interface import implementer


@implementer(IDefaultProtectionPolicy, ISiteRoot, ISiteRootRoles, IPortalContext,
             IIllustrationTarget, IHeaderTarget, IFooterTarget, IAlertTarget, IPreviewTarget)
class SiteRoot(ProtectedObject, BaseSiteRoot, UserSkinnableContent):
    """Main site root"""

    __roles__ = ('system.Manager', WEBMASTER_ROLE, OPERATOR_ROLE, 'pyams.TemplatesManager')

    roles_interface = ISiteRootRoles

    managers = RolePrincipalsFieldProperty(ISiteRootRoles['managers'])
    webmasters = RolePrincipalsFieldProperty(ISiteRootRoles['webmasters'])
    operators = RolePrincipalsFieldProperty(ISiteRootRoles['operators'])
    templates_managers = RolePrincipalsFieldProperty(ISiteRootRoles['templates_managers'])


@utility_config(provides=ISiteRootFactory)
class SiteRootFactory(object):
    """Default site root factory"""

    def __new__(cls):
        return SiteRoot


@implementer(ISiteRootConfiguration)
class SiteRootConfiguration(Configuration):
    """Site root configuration"""


@adapter_config(context=ISiteRoot, provides=IConfigurationFactory)
def site_root_configuration_factory(context):
    return SiteRootConfiguration


@implementer(ISiteRootBackOfficeConfiguration)
class SiteRootBackOfficeConfiguration(BackOfficeConfiguration):
    """Site root back-office configuration"""


@adapter_config(context=ISiteRoot, provides=IBackOfficeConfigurationFactory)
def site_root_back_office_configuration_factory(context):
    return SiteRootBackOfficeConfiguration


@subscriber(IGrantedRoleEvent)
def handle_granted_role(event):
    """Add principals to operators group when a role is granted"""
    role_id = event.role_id
    if (role_id == 'pyams.Operator') or (not role_id.startswith('pyams.')):
        return
    root = get_parent(event.object, ISiteRoot)
    if not root.operators:
        return
    security = get_utility(ISecurityManager)
    for principal_id in root.operators:
        if not principal_id:
            continue
        group = security.get_principal(principal_id, info=False)
        if event.principal_id not in group.principals:
            group.principals = group.principals | {event.principal_id}


@adapter_config(context=ISiteRoot, provides=IFormContextPermissionChecker)
class SiteRootPermissionChecker(ContextAdapter):
    """Site root permission checker"""

    @property
    def edit_permission(self):
        return MANAGE_SYSTEM_PERMISSION


#
# Tools configuration
#

@implementer(ISiteRootToolsConfiguration)
class SiteRootToolsConfiguration(Persistent):
    """Site root tools configuration"""

    tables_name = None

    pictograms_table_name = None

    tools_name = None

    views_tool_name = None
    forms_tool_name = None
    imagemaps_tool_name = None
    logos_tool_name = None
    news_tool_name = None


SITEROOT_TOOLS_CONFIGURATION_KEY = 'pyams_config.tools.configuration'


@adapter_config(context=ISiteRoot, provides=ISiteRootToolsConfiguration)
def site_root_tools_configuration_factory(context):
    """Site root tools configuration factory"""
    return get_annotation_adapter(context, SITEROOT_TOOLS_CONFIGURATION_KEY, SiteRootToolsConfiguration)
