#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import logging
logger = logging.getLogger("PyAMS (content)")

from datetime import datetime

# import interfaces
from hypatia.interfaces import ICatalog
from pyams_content.features.preview.interfaces import IPreviewTarget
from pyams_content.features.review.interfaces import IReviewTarget
from pyams_content.shared.view.interfaces import IView, IWfView, IWfViewFactory, IViewQuery, \
    IViewQueryParamsExtension, IViewQueryFilterExtension, VIEW_CONTENT_TYPE, VIEW_CONTENT_NAME, IViewSettings
from zope.intid.interfaces import IIntIds
from zope.lifecycleevent.interfaces import IObjectModifiedEvent

# import packages
from hypatia.catalog import CatalogQuery
from hypatia.query import Any, Gt, Lt
from pyams_cache.beaker import get_cache
from pyams_catalog.query import CatalogResultSet, or_
from pyams_content.shared.common import WfSharedContent, register_content_type, SharedContent, IWfSharedContentFactory
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.list import unique
from pyams_utils.registry import get_utility, get_global_registry
from pyams_utils.timezone import tztime
from pyams_workflow.interfaces import IWorkflow
from pyramid.events import subscriber
from pyramid.threadlocal import get_current_registry
from zope.interface import implementer, provider
from zope.schema.fieldproperty import FieldProperty


VIEWS_CACHE_REGION = 'views'
VIEWS_CACHE_NAME = 'PyAMS::view'

VIEW_CACHE_KEY = 'view_{view}'
VIEW_CONTEXT_CACHE_KEY = 'view_{view}.context_{context}'


@implementer(IWfView, IPreviewTarget, IReviewTarget)
class WfView(WfSharedContent):
    """Base view"""

    content_type = VIEW_CONTENT_TYPE
    content_name = VIEW_CONTENT_NAME

    select_context_type = FieldProperty(IWfView['select_context_type'])
    selected_content_types = FieldProperty(IWfView['selected_content_types'])
    order_by = FieldProperty(IWfView['order_by'])
    reversed_order = FieldProperty(IWfView['reversed_order'])
    limit = FieldProperty(IWfView['limit'])

    @property
    def is_using_context(self):
        if self.select_context_type:
            return True
        registry = get_global_registry()
        for name, adapter in registry.getAdapters((self,), IViewSettings):
            if not name:
                continue
            if adapter.is_using_context:
                return True
        return False

    def get_content_types(self, context):
        content_types = set()
        if self.select_context_type:
            content_types.add(context.content_type)
        if self.selected_content_types:
            content_types |= set(self.selected_content_types)
        return list(content_types)

    def get_results(self, context):
        intids = get_utility(IIntIds)
        views_cache = get_cache(VIEWS_CACHE_REGION, VIEWS_CACHE_NAME)
        if self.is_using_context:
            cache_key = VIEW_CONTEXT_CACHE_KEY.format(view=intids.queryId(self),
                                                      context=intids.queryId(context))
        else:
            cache_key = VIEW_CACHE_KEY.format(view=intids.queryId(self))
        try:
            results = views_cache.get_value(cache_key)
        except KeyError:
            registry = get_current_registry()
            adapter = registry.queryAdapter(self, IViewQuery, name='es')
            if adapter is None:
                adapter = registry.getAdapter(self, IViewQuery)
            results = adapter.get_results(context, self.limit)
            views_cache.set_value(cache_key, [intids.queryId(item) for item in results])
            logger.debug("Storing view items to cache key {0}".format(cache_key))
        else:
            results = CatalogResultSet(results)
            logger.debug("Retrieving view items from cache key {0}".format(cache_key))
        return results

register_content_type(WfView)


@provider(IWfViewFactory)
@implementer(IView)
class View(SharedContent):
    """Workflow managed view class"""


@adapter_config(context=IWfViewFactory, provides=IWfSharedContentFactory)
def view_content_factory(context):
    return WfView


@adapter_config(context=IWfView, provides=IViewQuery)
class ViewQuery(ContextAdapter):
    """View query"""

    def get_params(self, context):
        view = self.context
        catalog = get_utility(ICatalog)
        registry = get_current_registry()
        # check publication dates
        now = tztime(datetime.utcnow())
        params = Lt(catalog['effective_date'], now)
        params &= Gt(catalog['push_end_date'], now)
        # check content types
        content_types = view.get_content_types(context)
        if content_types:
            params &= Any(catalog['content_type'], content_types)
        # check workflow states
        wf_params = None
        for workflow in registry.getAllUtilitiesRegisteredFor(IWorkflow):
            wf_params = or_(wf_params, Any(catalog['workflow_state'], workflow.published_states))
        params &= wf_params
        # check custom extensions
        for name, adapter in sorted(registry.getAdapters((view,), IViewQueryParamsExtension),
                                    key=lambda x: x[1].weight):
            new_params = adapter.get_params(context)
            if new_params:
                params &= new_params
        return params

    def get_results(self, context, limit=None):
        view = self.context
        catalog = get_utility(ICatalog)
        registry = get_current_registry()
        params = self.get_params(context)
        items = CatalogResultSet(CatalogQuery(catalog).query(params,
                                                             sort_index=view.order_by,
                                                             reverse=view.reversed_order,
                                                             limit=limit))
        for name, adapter in sorted(registry.getAdapters((view,), IViewQueryFilterExtension),
                                    key=lambda x: x[1].weight):
            items = adapter.filter(context, items)
        return unique(items)


@subscriber(IObjectModifiedEvent, context_selector=IWfView)
def handle_modified_view(event):
    """Invalidate views cache when a view is modified"""
    view = event.object
    views_cache = get_cache(VIEWS_CACHE_REGION, VIEWS_CACHE_NAME)
    if view.is_using_context:
        views_cache.clear()
    else:
        intids = get_utility(IIntIds)
        cache_key = VIEW_CACHE_KEY.format(view=intids.queryId(view))
        views_cache.remove(cache_key)
